@@ -6,8 +6,9 @@ import numpy.random as rnd
 from alns.Result import Result
 from alns.State import State
 from alns.Statistics import Statistics
-from alns.criteria import AcceptanceCriterion
-from alns.weight_schemes import WeightScheme
+from alns.accept import AcceptanceCriterion
+from alns.stop import StoppingCriterion
+from alns.weights import WeightScheme
 
 # Potential candidate solution consideration outcomes.
 _BEST = 0
@@ -21,27 +22,27 @@ _OperatorType = Callable[[State, rnd.RandomState], State]
 
 
 class ALNS:
+    """
+    Implements the adaptive large neighbourhood search (ALNS) algorithm.
+    The implementation optimises for a minimisation problem, as explained
+    in the text by Pisinger and Røpke (2010).
+
+    Parameters
+    ----------
+    rnd_state
+        Optional random state to use for random number generation. When
+        passed, this state is used for operator selection and general
+        computations requiring random numbers. It is also passed to the
+        destroy and repair operators, as a second argument.
+
+    References
+    ----------
+    [1]: Pisinger, D., and Røpke, S. (2010). Large Neighborhood Search. In
+         M. Gendreau (Ed.), *Handbook of Metaheuristics* (2 ed., pp. 399
+         - 420). Springer.
+    """
 
     def __init__(self, rnd_state: rnd.RandomState = rnd.RandomState()):
-        """
-        Implements the adaptive large neighbourhood search (ALNS) algorithm.
-        The implementation optimises for a minimisation problem, as explained
-        in the text by Pisinger and Røpke (2010).
-
-        Parameters
-        ----------
-        rnd_state
-            Optional random state to use for random number generation. When
-            passed, this state is used for operator selection and general
-            computations requiring random numbers. It is also passed to the
-            destroy and repair operators, as a second argument.
-
-        References
-        ----------
-        [1]: Pisinger, D., and Røpke, S. (2010). Large Neighborhood Search. In
-             M. Gendreau (Ed.), *Handbook of Metaheuristics* (2 ed., pp. 399
-             - 420). Springer.
-        """
         self._destroy_operators: Dict[str, _OperatorType] = {}
         self._repair_operators: Dict[str, _OperatorType] = {}
 
@@ -107,12 +108,14 @@ class ALNS:
         """
         self._repair_operators[name if name else op.__name__] = op
 
-    def iterate(self,
-                initial_solution: State,
-                weight_scheme: WeightScheme,
-                crit: AcceptanceCriterion,
-                iterations: int = 10_000,
-                **kwargs) -> Result:
+    def iterate(
+        self,
+        initial_solution: State,
+        weight_scheme: WeightScheme,
+        crit: AcceptanceCriterion,
+        stop: StoppingCriterion,
+        **kwargs,
+    ) -> Result:
         """
         Runs the adaptive large neighbourhood search heuristic [1], using the
         previously set destroy and repair operators. The first solution is set
@@ -129,8 +132,10 @@ class ALNS:
         crit
             The acceptance criterion to use for candidate states. See also
             the ``alns.criteria`` module for an overview.
-        iterations
-            The number of iterations. Default 10_000.
+        stop
+            The stopping criterion to use for stopping the iterations.
+            See also the ``alns.stopping_criteria`` module for an overview.
+
         **kwargs
             Optional keyword arguments. These are passed to the operators,
             including callbacks.
@@ -156,10 +161,9 @@ class ALNS:
         Operational Research*, 171: 750–775, 2006.
         """
         if len(self.destroy_operators) == 0 or len(self.repair_operators) == 0:
-            raise ValueError("Missing at least one destroy or repair operator.")
-
-        if iterations < 0:
-            raise ValueError("Negative number of iterations.")
+            raise ValueError(
+                "Missing at least one destroy or repair operator."
+            )
 
         curr = best = initial_solution
 
@@ -167,7 +171,7 @@ class ALNS:
         stats.collect_objective(initial_solution.objective())
         stats.collect_runtime(time.perf_counter())
 
-        for iteration in range(iterations):
+        while not stop(self._rnd_state, best, curr):
             d_idx, r_idx = weight_scheme.select_operators(self._rnd_state)
 
             d_name, d_operator = self.destroy_operators[d_idx]
@@ -176,11 +180,9 @@ class ALNS:
             destroyed = d_operator(curr, self._rnd_state, **kwargs)
             cand = r_operator(destroyed, self._rnd_state, **kwargs)
 
-            best, curr, s_idx = self._eval_cand(crit,
-                                                best,
-                                                curr,
-                                                cand,
-                                                **kwargs)
+            best, curr, s_idx = self._eval_cand(
+                crit, best, curr, cand, **kwargs
+            )
 
             weight_scheme.update_weights(d_idx, r_idx, s_idx)
 
@@ -206,12 +208,12 @@ class ALNS:
         self._on_best = func
 
     def _eval_cand(
-            self,
-            crit: AcceptanceCriterion,
-            best: State,
-            curr: State,
-            cand: State,
-            **kwargs
+        self,
+        crit: AcceptanceCriterion,
+        best: State,
+        curr: State,
+        cand: State,
+        **kwargs,
     ) -> Tuple[State, State, int]:
         """
         Considers the candidate solution by comparing it against the best and
